using System;
using System.Collections;

namespace BinaryTree
{
	public class Action
	{
		public virtual void Execute(Tree node)	{}
	};

	public class DeleteAction : Action
	{
		override public void Execute(Tree node)
		{
			node.LeftTree = null;
			node.RightTree = null;
			node.NodeData = null;
			// the garbage collector will
			// take out the trash :)
		}
	}


	public class Tree
    {
		private static int count = 0;
		private static int depth = 0;

		private object data;
        private Tree left;
		private Tree right;

		public Tree()
		{
			count = 0;
			depth = 0;
			this.data = null;
			this.left = null;
			this.right = null;
		}

		public Tree(object nodeValue)
		{
			this.data = nodeValue;
			this.left = null;
			this.right = null;
		}

		public object NodeData
		{
			get { return this.data; }
			set { this.data = value; }
		}

        public Tree LeftTree
        {
            get { return this.left; }
            set { this.left = value; }
        }

        public Tree RightTree
        {
            get { return this.right; }
            set { this.right = value; }
        }

		public void Empty()
		{
			DeleteAction myDeleteAction = new DeleteAction();
			PostOrder(myDeleteAction);
			count = 0; depth = 0;
		}

		public int GetDepth()
		{
			return depth;
		}

		public int GetCount()
		{
			return count;
		}

		public void Insert(object newItem)
		{
			// simulating default parameter
			Insert(newItem, 1);
		}

        private void Insert(object newItem, int myDepth)
        {
			if (this.NodeData != null)
			{
				object currentNodeValue = this.NodeData;
				if (Comparer.DefaultInvariant.Compare(currentNodeValue, newItem) > 0)
				{
					if (this.LeftTree == null)
					{
						if (myDepth > depth) depth=myDepth;	count++;
						this.LeftTree = new Tree(newItem);
					}
					else
					{
						this.LeftTree.Insert(newItem, myDepth+1);
					}
				}
				else
				{
					if (this.RightTree == null)
					{
						if (myDepth > depth) depth=myDepth;	count++;
						this.RightTree = new Tree(newItem);
					}
					else
					{
						this.RightTree.Insert(newItem, myDepth+1);
					}
				}
			}
			else
			{
				this.NodeData = newItem;
				depth++; count++;
			}
        }

		public int MustRebuild()
		{
			double temp = 2*Math.Log((double)count, 2.0);
			// this will be useful I hope so
			if ( (depth > temp) && (temp != 0) )
			{
				return count;
			}
			else
			{
				return -1;
			}
		}

        public void PreOrder(Action myAction)
        {
			myAction.Execute(this);
            if (this.LeftTree != null)
            {
                this.LeftTree.InOrder(myAction);
            }
            if (this.RightTree != null)
            {
                this.RightTree.InOrder(myAction);
            }
		}

		public void InOrder(Action myAction)
		{
			if (this.LeftTree != null)
			{
				this.LeftTree.InOrder(myAction);
			}
			myAction.Execute(this);
			if (this.RightTree != null)
			{
				this.RightTree.InOrder(myAction);
			}
		}

		public void PostOrder(Action myAction)
		{
			if (this.LeftTree != null)
			{
				this.LeftTree.InOrder(myAction);
			}
			if (this.RightTree != null)
			{
				this.RightTree.InOrder(myAction);
			}
			myAction.Execute(this);
		}

	};

	
	public class TreeDrawer
	{

		public void Draw (System.Drawing.Graphics g, Tree myTree)
		{
			System.Drawing.Font myFont = new System.Drawing.Font(new System.Drawing.FontFamily("Arial"), 12, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Pixel);
			RecursiveDraw(myTree, g, 0, 1, 0);
		}

		private void RecursiveDraw (Tree myTree, System.Drawing.Graphics g, int depth, int arrange, int position)
		{
			if (myTree != null)
			{
				double x = position + (g.ClipBounds.Right / System.Math.Pow(2, depth+1)) * arrange;
				double y = 30 + (depth) * 40;
				if (position != 0)
				{
					g.DrawLine(new System.Drawing.Pen(System.Drawing.Color.CornflowerBlue, 2), position, (int)y-40, (int)x, (int)y);
				}
				RecursiveDraw(myTree.LeftTree, g, depth+1, -1, (int)x);
				RecursiveDraw(myTree.RightTree, g, depth+1, +1, (int)x);
				g.DrawEllipse(new System.Drawing.Pen(System.Drawing.Color.Black, 2), (int)x-10, (int)y-10, 20, 20);
				g.FillEllipse(new System.Drawing.SolidBrush(System.Drawing.Color.DarkBlue), (int)x-10, (int)y-10, 20, 20);
				System.Drawing.Font myFont = new System.Drawing.Font(new System.Drawing.FontFamily("Arial"), 12, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Pixel);
				g.DrawString(myTree.NodeData.ToString(), myFont, new System.Drawing.SolidBrush(System.Drawing.Color.Black), (float)x-(6*myTree.NodeData.ToString().Length), (float)y-25);
			}
		}
	};


	public class TreeRebuilder
	{
		private object[] treeList;
		private Tree myTree;

		private class ParseAction : Action
		{
			private int i;
			private object[] myList;
			public ParseAction(ref object[] myList)
			{
				this.i = 0;
				this.myList = myList;
			}
			override public void Execute(Tree node)
			{
				myList[i++] = node.NodeData;
			}
		}

		public TreeRebuilder(ref Tree tree)
		{
			this.myTree = tree;
			treeList = new object [ myTree.GetCount() ];
		}

		public TreeRebuilder(ref Tree tree, bool execute)
		{
			this.myTree = tree;
			treeList = new object [ myTree.GetCount() ];
			if (execute) Rebuild();
		}

		public void Rebuild()
		{
			ParseAction myParseAction = new ParseAction(ref treeList);
			myTree.InOrder(myParseAction);
			myTree.Empty();
			LogaritmicBuild(0, treeList.Length-1, ref treeList);
		}

		private void LogaritmicBuild (int u, int v, ref object[] myList)
		{
			int i = (u+v)/2;
			if ( (i!=u) && (i!=v) )
			{
				this.myTree.Insert(myList[i]);
				LogaritmicBuild(u, i-1, ref myList);
				LogaritmicBuild(i+1, v, ref myList);
			}
			else if ( u!=v )
			{
				this.myTree.Insert(myList[u]);
				this.myTree.Insert(myList[v]);
			}
			else
			{
				this.myTree.Insert(myList[u]);
			}
		}
	}
}
